/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/trunk/backends/platform/iphone/iphone_main.m $
 * $Id: iphone_main.m 40872 2009-05-24 21:53:29Z lordhoto $
 *
 */
#include <sys/types.h>
#include <sys/sysctl.h>
#include <semaphore.h>
#import "switchViewController.h"
#import "iPhoneViewController.h"
#import "Gob1MenuViewController.h"
#import "menuigViewController.h"
#import "DotEmuChannelViewController.h"
#import "iphone_video.h"
#import "iphone_main.h"
#import "iphone_common.h"
#import <UIKit/UIKit.h>
#import <Foundation/NSThread.h>
#import <AudioToolbox/AudioServices.h>


//////////////////
//---GAME PARAMS
//LVL (SIMULATION)
extern bool bSimulation;
extern bool bLvLLoaded;
extern int idLvLCourant;
//GAME (GENERAL)
extern bool bHintMenuActive;
extern bool bMenuIngame;
extern bool bFromMenuIngame;
extern bool bAnimBeginning;
extern bool bMenuGob1;
bool bBackground = false;
//SOUND
extern bool bSoundMusicActive;
extern int idSoundMusic;
extern int idSoundSFX;
extern AVAudioPlayer* menuPlayer;
UInt32 isiPodPlaying;
//VM
bool bSleepMode = false;
//DEFINES
extern void initScumm(int argc, char ** argv);
extern iPhoneViewController* g_iVC;
extern switchViewController* g_switchVC;
extern Gob1MenuViewController* g_gob1MenuVC;
iPhoneMain* _delegate;
static int gArgc;
static char** gArgv;
//---END
////////////////

extern bool bSem;
extern sem_t* sem;
int _GobLanguage = 0;

//platform
NSString* _plat;

int main(int argc, char** argv) 
{
	gArgc = argc;
	gArgv = argv;
	
    NSAutoreleasePool* autoreleasePool = [[NSAutoreleasePool alloc] init];
	
    int returnCode = UIApplicationMain(argc, argv, @"UIApplication", @"iPhoneMain");
    [autoreleasePool release];
    return returnCode;
}

@implementation iPhoneMain
@synthesize _DEChannel, _iVC, _menuigVC, _switchVC, _gob1MenuVC, _view;

- (NSString *) platform
{
	size_t size;
	sysctlbyname("hw.machine", NULL, &size, NULL, 0);
	char *machine = (char*)malloc(size);
	sysctlbyname("hw.machine", machine, &size, NULL, 0);
	NSString *platformCode = [[NSString stringWithCString:machine encoding:NSUTF8StringEncoding] retain];
	free(machine);
	return platformCode;
}

//ANIMATIONS
-(void)splashViewFadeIn
{
	_switchVC.view.alpha = 0;
	[UIView beginAnimations:nil context:self]; 
	{
		[UIView setAnimationDuration:0.35];
		[UIView setAnimationDelegate:self];
		[UIView setAnimationCurve:UIViewAnimationCurveLinear];
		_switchVC.view.alpha = 1;
	}
	[UIView commitAnimations];
}

-(void)splashViewFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context 
{
	/*if( viewController != nil)
	 {
	 [viewController.view removeFromSuperview];
	 [viewController release];
	 viewController = nil;
	 }*/
}

-(void)splashViewFadeOut
{
	_switchVC.view.alpha = 1;
	[UIView beginAnimations:nil context:self]; 
	{
		[UIView setAnimationDuration:0.35];
		[UIView setAnimationDelegate:self];
		[UIView setAnimationCurve:UIViewAnimationCurveLinear];
		[UIView setAnimationDidStopSelector:@selector(splashViewFinished:finished:context:)];
		_switchVC.view.alpha = 0;
	}
	[UIView commitAnimations];
}

-(void)splashDotEmuStoreFadeIn
{
	_DEChannel.view.alpha = 0;
	[UIView beginAnimations:nil context:self]; 
	{
		[UIView setAnimationDuration:0.35];
		[UIView setAnimationDelegate:self];
		[UIView setAnimationCurve:UIViewAnimationCurveLinear];
		_DEChannel.view.alpha = 1;
	}
	[UIView commitAnimations];
}

-(void)splashDotEmuStoreFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context 
{
	if( _DEChannel != nil)
	{
		[_DEChannel.view removeFromSuperview];
		[_DEChannel release];
		_DEChannel = nil;
	}
}

-(void)splashDotEmuStoreFadeOut
{
	_DEChannel.view.alpha = 1;
	[UIView beginAnimations:nil context:self]; 
	{
		[UIView setAnimationDuration:0.35];
		[UIView setAnimationDelegate:self];
		[UIView setAnimationCurve:UIViewAnimationCurveLinear];
		[UIView setAnimationDidStopSelector:@selector(splashDotEmuStoreFinished:finished:context:)];		
		_DEChannel.view.alpha = 0;
	}
	[UIView commitAnimations];
}

- (void) switchToPortraitView
{
	[self splashViewFadeOut];
	
	_DEChannel = [[DotEmuChannelViewController alloc] init];
	_DEChannel.view.alpha = 0;
	[_window addSubview:_DEChannel.view];
	
	[NSTimer scheduledTimerWithTimeInterval:0 target:self selector:@selector(splashDotEmuStoreFadeIn) userInfo:nil repeats:NO];	
}

- (void) switchToLandscapeView
{
	[self splashDotEmuStoreFadeOut];
	
	//viewController = [[MainViewController alloc] init];
	_switchVC.view.alpha = 0;	
	//[window addSubview:viewController.view];
	
	[NSTimer scheduledTimerWithTimeInterval:0 target:self selector:@selector(splashViewFadeIn) userInfo:nil repeats:NO];		
}


-(id) init 
{
	//NSLog(@"iPhoneMain init");
	[super init];
	_window = nil;
	_view = nil;
	_delegate = self;
	
	//LANGUAGE
	NSArray* languages = [NSLocale preferredLanguages];
    NSString* preferredLang = [languages objectAtIndex:0];
	if( [preferredLang isEqualToString:@"en"] )
		_GobLanguage = 0;
	else if( [preferredLang isEqualToString:@"fr"] )
		_GobLanguage = 1;
	else
		_GobLanguage = 0;
	
	//SAVE INFOS LvL
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path_sound = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_SOUND.DAT"]];
	
	/*NSFileManager* fileManager = [NSFileManager defaultManager];
	[fileManager removeItemAtPath:path error:NULL];
	[fileManager removeItemAtPath:path_sound error:NULL];*/
	
	NSString *fileString = [NSString stringWithContentsOfFile:path_sound encoding:NSUTF8StringEncoding error:nil];
	NSArray *mArray = [fileString componentsSeparatedByString:@"\n"];
	NSString *ll = mArray.lastObject;

	if( ll != nil )
	{
		idSoundMusic = [[ll substringWithRange:NSMakeRange(6, 3)] intValue];
		if( idSoundMusic >= 100 )
			idSoundSFX = [[ll substringWithRange:NSMakeRange(14, 3)] intValue];
		else if( idSoundMusic < 100 && idSoundMusic >= 10 )
			idSoundSFX = [[ll substringWithRange:NSMakeRange(13, 3)] intValue];		
		else
			idSoundSFX = [[ll substringWithRange:NSMakeRange(12, 3)] intValue];
	}
	
	return self;
}

- (void) mainLoop: (id)param
{
	//NSLog(@"iPhoneMain mainLoop");
	[[NSAutoreleasePool alloc] init];

	_view = (iPhoneView*)g_iVC.view;

	initScumm(gArgc,gArgv);
	
	[_view applicationSuspend];	
}

- (iPhoneView*) getView
{
	//NSLog(@"iPhoneMain getView");	
	return _view;
}

- (void) setView:(iPhoneView*)iView
{
	//NSLog(@"iPhoneMain setView");		
	_view = iView;
}

- (void)applicationWillterminate:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationWillterminate");
	if ( g_iVC.view.superview != nil && g_iVC != nil ) 
		[(iPhoneView*)g_iVC.view applicationSuspend];	
	
	[_view release];
    [_iVC release];
    [_gob1MenuVC release];
    [_switchVC release];
    [_window release];
}

- (void) pauseGame
{
	//NSLog(@"iPhoneMain pauseGame");	
	[(iPhoneView*)g_iVC.view addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonEscapeDown], @"type",
						  [NSNumber numberWithFloat:0.0f], @"x",[NSNumber numberWithFloat:0.0f], @"y",nil]];
	
	[(iPhoneView*)g_iVC.view addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonEscapeUp], @"type",
									   [NSNumber numberWithFloat:0.0f], @"x",[NSNumber numberWithFloat:0.0f], @"y",nil]];
}

-(void)applicationWillResignActive:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationWillResignActive");	
	if( [[UIDevice currentDevice].systemVersion floatValue] > 3.2)
	{
		bBackground = true;
		if( g_iVC.view.superview != nil && g_iVC != nil ) 
			[(iPhoneView*)g_iVC.view applicationSuspend];	
	}
	
	if ( g_iVC.view.superview != nil && g_iVC != nil )
	{
		if( !bAnimBeginning && bLvLLoaded && !bSimulation && idLvLCourant != -2 && !bMenuIngame && !bFromMenuIngame && !bHintMenuActive )
		{
			bSleepMode = true;
			[g_switchVC switchPause];
			bMenuIngame = true;
		}
	}
		
	[[NSNotificationCenter defaultCenter] postNotificationName:@"muteNotif" object:nil];
	if( menuPlayer != nil )
		[menuPlayer stop];
	if( bMenuGob1 )	
		[[NSNotificationCenter defaultCenter] postNotificationName:@"pauseAnimNotif" object:nil];	
}

- (void) applicationDidBecomeActive:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationDidBecomeActive");		
	if( [[UIDevice currentDevice].systemVersion floatValue] > 3.2)
	{
		bBackground = false;
		if ( g_iVC.view.superview != nil && g_iVC != nil )
			[_view performSelectorOnMainThread:@selector(updateSurface) withObject:nil waitUntilDone: NO];
	}
	
	if( !bMenuIngame && !bHintMenuActive )
	{
		bSleepMode = false;
	}
	
	//resuming semaphore
	if( bSem && !bMenuIngame && !bHintMenuActive )
	{
		NSLog(@"resume sempahore");
		sem_post( sem );		
		bSem = false;
	}
	
	UInt32 propertySize = sizeof(isiPodPlaying);	
	status = AudioSessionGetProperty(kAudioSessionProperty_OtherAudioIsPlaying, &propertySize, &isiPodPlaying);
	
	if( isiPodPlaying )
	{
		UInt32 sessionCategory = kAudioSessionCategory_AmbientSound;
		status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory);
		bSoundMusicActive = false;		
		idSoundMusic = 0;	
	}
	else
	{
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory); 
		bSoundMusicActive = true;
		[[NSNotificationCenter defaultCenter] postNotificationName:@"settingsMusicNotif" object:nil];

		if( !bMenuGob1 )
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:@"setSoundsNotif" object:nil];
		}
		else 
		{
			if( menuPlayer != nil )
			{
				[menuPlayer stop];
				menuPlayer.volume = (float)((float)idSoundMusic / 256.0f);
				menuPlayer.numberOfLoops = -1;
				if( g_gob1MenuVC.view.alpha == 1 )
				{
					[menuPlayer play];
				}

			}
		}
	}
	
	if( bMenuGob1 )
		[[NSNotificationCenter defaultCenter] postNotificationName:@"resumeAnimNotif" object:nil];	
}

-(void)applicationDidEnterBackground:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationDidEnterBackground");	
	if( bMenuGob1 )	
		[[NSNotificationCenter defaultCenter] postNotificationName:@"stopAnimNotif" object:nil];
}

-(void)applicationWillEnterForeground:(UIApplication *)application
{
	//NSLog(@"iPhoneMain applicationWillEnterForeground");
	UInt32 propertySize = sizeof(isiPodPlaying);	
	status = AudioSessionGetProperty(kAudioSessionProperty_OtherAudioIsPlaying, &propertySize, &isiPodPlaying);	
	NSURL *url = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/music_menu_48.mp3", [[NSBundle mainBundle] resourcePath]]];
	NSError *err;
	if( idSoundMusic == 0 )
		idSoundMusic = 192;
	
	if( !isiPodPlaying )
	{
		if( !bMenuGob1 )
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:@"setSoundsNotif" object:nil];
		}
		else 
		{
			if( menuPlayer != nil )
			{
				[menuPlayer stop];
				menuPlayer.volume = (float)((float)idSoundMusic / 256.0f);
				menuPlayer.numberOfLoops = -1;
				[menuPlayer play];	
			}
			else 
			{
				menuPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:&err];			 
				[menuPlayer stop];
				menuPlayer.volume = (float)((float)idSoundMusic / 256.0f);
				menuPlayer.numberOfLoops = -1;
				[menuPlayer play];
			}
			
		}
	}	
	
	if( bMenuGob1 )	
		[[NSNotificationCenter defaultCenter] postNotificationName:@"startAnimNotif" object:nil];		
}

- (void)applicationDidFinishLaunching:(UIApplication *)application 
{
	//NSLog(@"iPhoneMain applicationDidFinishLaunching");	
	application.idleTimerDisabled = YES;
	CGRect  rect = [[UIScreen mainScreen] bounds];
	
	//if (NSClassFromString(@"iSimulate"))
	//	[NSClassFromString(@"iSimulate") disableTouchesOverlay];
	
	_plat = [self platform];

	// hide the status bar
    [application setStatusBarStyle:UIStatusBarStyleBlackTranslucent animated:NO];
    [application setStatusBarHidden:YES withAnimation:UIStatusBarAnimationNone];

	_window = [[UIWindow alloc] initWithFrame:rect];
	[_window retain];

	//<TEST AUDIO SESSION
	UInt32 propertySize = sizeof(isiPodPlaying);	
	AudioSessionInitialize(NULL, NULL, NULL, self);
	status = AudioSessionGetProperty(kAudioSessionProperty_OtherAudioIsPlaying, &propertySize, &isiPodPlaying);
	//TEST AUDIO SESSION>
	
	//if it's playing, music volume control = 0
	if( isiPodPlaying )
	{
		UInt32 sessionCategory = kAudioSessionCategory_AmbientSound;
		status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory);	
		bSoundMusicActive = false;		
		idSoundMusic = 0;	
	}
	else
	{
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		status = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(sessionCategory), &sessionCategory); 
		bSoundMusicActive = true;	
	}
	
	[self launchViews];
}

-(void)launchViews
{
	//NSLog(@"iPhoneMain launchViews");	
	[[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillterminate:) name:UIApplicationWillTerminateNotification object:nil];
	
	//go to SWITCH VIEW 
	_switchVC = [[switchViewController alloc] initWithNibName:@"switchViewController" bundle:[NSBundle mainBundle]];
	//_view = _switchVC.view; 
	_switchVC.view.multipleTouchEnabled = NO;
	
	[_window addSubview:_switchVC.view];
	[_window makeKeyAndVisible];
}

-(void)rotationNotification
{
	//NSLog(@"iPhoneMain rotationNotification");		
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(didRotate:)
												 name:@"UIDeviceOrientationDidChangeNotification" object:nil];
}

- (void)didRotate:(NSNotification *)notification
{
	//NSLog(@"iPhoneMain didRotate");			
	UIDeviceOrientation screenOrientation = [[UIDevice currentDevice] orientation]; 
	
	if ( g_iVC.view.superview != nil && g_iVC != nil ) 
		[_view deviceOrientationChanged: screenOrientation];
}

- (UIWindow*) getWindow 
{
	//NSLog(@"iPhoneMain getWindow");		
	return _window;
}

@end

